@@ -16,7 +16,7 @@
 # Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 # MA 02110-1301, USA, or see http://www.gnu.org/.
 
-VERSION = "2.15"
+VERSION = "2.18"
 HTTP_AGENT = "rawdog/" + VERSION
 STATE_VERSION = 2
 
@@ -198,14 +198,14 @@ def author_to_html(entry, feedurl, config):
 	url = None
 	fallback = "author"
 	if author_detail is not None:
-		if author_detail.has_key("url"):
-			url = author_detail["url"]
+		if author_detail.has_key("href"):
+			url = author_detail["href"]
 		elif author_detail.has_key("email") and author_detail["email"] is not None:
 			url = "mailto:" + author_detail["email"]
 		if author_detail.has_key("email") and author_detail["email"] is not None:
 			fallback = author_detail["email"]
-		elif author_detail.has_key("url") and author_detail["url"] is not None:
-			fallback = author_detail["url"]
+		elif author_detail.has_key("href") and author_detail["href"] is not None:
+			fallback = author_detail["href"]
 
 	if name == "":
 		name = fallback
@@ -265,12 +265,24 @@ def fill_template(template, bits):
 
 file_cache = {}
 def load_file(name):
-	"""Read the contents of a file, caching the result so we don't have to
-	read the file multiple times."""
+	"""Read the contents of a template file, caching the result so we don't
+	have to read the file multiple times. The file is assumed to be in the
+	system encoding; the result will be an ASCII string."""
 	if not file_cache.has_key(name):
-		f = open(name)
-		file_cache[name] = f.read()
-		f.close()
+		try:
+			f = open(name)
+			data = f.read()
+			f.close()
+		except IOError:
+			raise ConfigError("Can't read template file: " + name)
+
+		try:
+			data = data.decode(get_system_encoding())
+		except UnicodeDecodeError, e:
+			raise ConfigError("Character encoding problem in template file: " + name + ": " + str(e))
+
+		data = encode_references(data)
+		file_cache[name] = data.encode(get_system_encoding())
 	return file_cache[name]
 
 def write_ascii(f, s, config):
@@ -701,7 +713,7 @@ class DayWriter:
 	"""Utility class for writing day sections into a series of articles."""
 
 	def __init__(self, file, config):
-		self.lasttime = [-1, -1, -1, -1, -1]
+		self.lasttime = []
 		self.file = file
 		self.counter = 0
 		self.config = config
@@ -842,6 +854,7 @@ class Config:
 			}
 
 	def __getitem__(self, key): return self.config[key]
+	def get(self, key, default=None): return self.config.get(key, default)
 	def __setitem__(self, key, value): self.config[key] = value
 
 	def reload(self):
@@ -859,7 +872,12 @@ class Config:
 		try:
 			f = open(filename, "r")
 			for line in f.xreadlines():
-				stripped = line.decode(get_system_encoding()).strip()
+				try:
+					line = line.decode(get_system_encoding())
+				except UnicodeDecodeError, e:
+					raise ConfigError("Character encoding problem in config file: " + filename + ": " + str(e))
+
+				stripped = line.strip()
 				if stripped == "" or stripped[0] == "#":
 					continue
 				if line[0] in string.whitespace:
@@ -887,6 +905,11 @@ class Config:
 		elif len(l) != 2:
 			raise ConfigError("Bad line in config: " + line)
 
+		# Load template files immediately, so we produce an error now
+		# rather than later if anything goes wrong.
+		if l[0].endswith("template") and l[1] != "default":
+			load_file(l[1])
+
 		handled_arglines = False
 		if l[0] == "feed":
 			l = l[1].split(None)
@@ -1126,12 +1149,14 @@ class FeedState(Persistable):
 	"""The collection of articles in a feed."""
 
 	def __init__(self):
+		Persistable.__init__(self)
 		self.articles = {}
 
 class Rawdog(Persistable):
 	"""The aggregator itself."""
 
 	def __init__(self):
+		Persistable.__init__(self)
 		self.feeds = {}
 		self.articles = {}
 		self.plugin_storage = {}
@@ -1214,7 +1239,7 @@ class Rawdog(Persistable):
 		# Convert to or from splitstate if necessary.
 		try:
 			u = self.using_splitstate
-		except:
+		except AttributeError:
 			# We were last run with a version of rawdog that didn't
 			# have this variable -- so we must have a single state
 			# file.
@@ -1241,8 +1266,7 @@ class Rawdog(Persistable):
 							self.articles[article_hash] = article
 						feedstate.articles = {}
 						feedstate.modified()
-					# FIXME: Have Persisted do this?
-					os.unlink(feed.get_state_filename())
+					persister.delete(feed.get_state_filename())
 			self.modified()
 			self.using_splitstate = config["splitstate"]
 
@@ -1269,10 +1293,7 @@ class Rawdog(Persistable):
 			if url not in seen_feeds:
 				config.log("Removing feed: ", url)
 				if config["splitstate"]:
-					try:
-						os.unlink(self.feeds[url].get_state_filename())
-					except OSError:
-						pass
+					persister.delete(self.feeds[url].get_state_filename())
 				else:
 					for key, article in self.articles.items():
 						if article.feed == url:
@@ -1382,7 +1403,7 @@ class Rawdog(Persistable):
 	def get_template(self, config, name="page"):
 		"""Return the contents of a template."""
 
-		filename = config[name + "template"]
+		filename = config.get(name + "template", "default")
 		if filename != "default":
 			return load_file(filename)
 
@@ -1454,11 +1475,14 @@ __feeditems__
 </tr>
 """
 		else:
-			raise ValueError("Unknown template name: " + name)
+			raise KeyError("Unknown template name: " + name)
 
 	def show_template(self, name, config):
 		"""Show the contents of a template, as currently configured."""
-		print self.get_template(config, name),
+		try:
+			print self.get_template(config, name),
+		except KeyError:
+			print >>sys.stderr, "Unknown template name: " + a
 
 	def write_article(self, f, article, config):
 		"""Write an article to the given file."""
